<?php

namespace Drupal\json_table\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Render\RendererInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'json_editor_widget' widget.
 *
 * @FieldWidget(
 *   id = "json_table_widget",
 *   label = @Translation("Json table"),
 *   field_types = {
 *     "json",
 *   }
 * )
 */
class JsonTableWidget extends WidgetBase {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, RendererInterface $renderer) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('renderer'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'header' => '',
      'ajax' => FALSE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritDoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $field_name = $items->getName();
    $id = Html::getUniqueId($field_name . '-' . $delta);
    // Decode JSON data only if AJAX is enabled.
    $valuesRow = [];
    $value = $items[$delta]?->value ?? NULL;
    if (!$this->getSetting('ajax') && !empty($items[$delta]->value)) {
      $valuesRow = json_decode($items[$delta]->value, TRUE);
      // Sometimes we get data from X-speadsheet.
      if (!empty($valuesRow['rows'])) {
        $valuesRow = $this->convertSpreadSheet($valuesRow['rows']);
        $value = json_encode($valuesRow);
      }
    }
    // Render table container.
    $container = $this->renderTable($id, $valuesRow);

    // Render the container if AJAX is enabled.
    if ($this->getSetting('ajax')) {
      $container = Markup::create($this->renderer->render($container));
    }
    $elements['value'] = [
      '#title' => $this->fieldDefinition->getLabel(),
      '#type' => 'textarea',
      '#default_value' => $value,
      '#description' => $container,
      '#attributes' => [
        'data-json-table' => $this->getSetting('mode'),
        'class' => ['json-table', 'js-hide', $this->getSetting('mode')],
      ],
      '#attached' => [
        'library' => ['json_table/json_table'],
        'drupalSettings' => [
          'json_editor' => [$delta => $this->getSetting('mode')],
        ],
      ],
      '#element_validate' => [
        [$this, 'validateJsonData'],
      ],
    ];
    return $elements;
  }

  /**
   * {@inheritDoc}
   */
  public function renderTable($id, $data) {
    $textPaste = $this->t('Paste your Excel data here');
    $header = [];
    $checkJson = json_decode($this->getSetting('header'), TRUE);
    if (!$checkJson && !empty($this->getSetting('header'))) {
      $header = str_replace([
        ',',
        ';',
        "\r\n",
        "\r",
        "\t",
      ], PHP_EOL, $this->getSetting('header'));
      $header = explode(PHP_EOL, $header);
    }
    elseif (is_array($checkJson)) {
      $header = $checkJson;
    }
    $container = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [
          'json-table-wrapper',
        ],
      ],
      'btn' => [
        '#markup' => Markup::create(
          '<button type="button" class="button button-action button--primary btn btn-primary add-row">' . $this->t('Add row') . '</button>
<button type="button" class="button button-action button--primary btn btn-primary add-col">' . $this->t('Add column') . '</button>
<button type="button" class="button button--danger btn btn-danger remove-row">' . $this->t('Remove row') . '</button>
<button type="button" class="button button--danger btn btn-danger remove-col">' . $this->t('Remove column') . '</button>'),
      ],
      'table' => [
        '#type' => 'table',
        '#header' => $header,
        '#attributes' => [
          'id' => $id,
          'class' => [
            'json-table',
          ],
        ],
      ],
    ];
    if (empty($data)) {
      $inputRow = [
        '#type' => 'textfield',
        '#title' => '',
        '#title_display' => 'invisible',
        '#attributes' => [
          'class' => ['paste', 'form-control', 'form-text', 'form-element'],
          'placeholder' => $textPaste,
        ],
        '#maxlength' => NULL,
        '#size' => NULL,
      ];
      $rowDefault = [$inputRow];
      if (!empty($header) && count($header) > 1) {
        unset($inputRow['#attributes']['placeholder']);
        foreach (range(1, count($header) - 1) as $col) {
          $rowDefault[] = $inputRow;
        }
      }
      $container['table'] += [$rowDefault];
    }
    else {
      foreach ($data as $x => $row) {
        if (!is_array($row)) {
          continue;
        }
        foreach ($row as $y => $col) {
          $type = is_numeric($col) ? 'number' : 'textfield';
          $container['table'][$x][$y] = [
            '#type' => $type,
            '#title' => '',
            '#title_display' => 'invisible',
            '#value' => is_string($col) ? $col : '',
            '#attributes' => [
              'class' => ['form-control', 'form-text', 'form-element'],
            ],
          ];
        }
      }
    }
    return $container;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements['header'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Defined header'),
      '#description' => $this->t('You can set the header separated by tab, comma ",", semicolon ";" or with a json object if you want switch to json viewer, for example:') .
      json_encode(['name' => "Col Name", 'age' => 'Col Age']),
      '#default_value' => $this->getSetting('header'),
    ];
    $elements['ajax'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Ajax load'),
      '#description' => $this->t('For high performance, use if you have a large JSON table'),
      '#default_value' => $this->getSetting('ajax'),
    ];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    if (!empty($this->getSetting('header'))) {
      $summary[] = $this->t('Custom header');
    }
    if (!empty($this->getSetting('ajax'))) {
      $summary[] = $this->t('Load data table with javascript');
    }
    return $summary;
  }

  /**
   * Check the submitted JSON against the configured JSON Schema.
   *
   * {@inheritdoc}
   */
  public static function validateJsonData($element, FormStateInterface $form_state) {
    json_decode($element['#value']);
    return json_last_error() == JSON_ERROR_NONE;
  }

  /**
   * Convert data type XspreadSheet to array data.
   *
   * {@inheritdoc}
   */
  protected function convertSpreadSheet(mixed $rows): array {
    $data = [];
    foreach ($rows as $row) {
      if (!is_array($row)) {
        continue;
      }
      $cells = [];
      foreach ($row['cells'] as $cell) {
        $cells[] = $cell['text'];
      }
      if (!empty($cells)) {
        $data[] = $cells;
      }
    }
    return $data;
  }

}
